//
//  APIClient.swift
//  Notes
//
//  Created by Roman Vyjídáček on 17.04.2024.
//

import Foundation

final class APIClient {
    
    static private(set) var shared = APIClient()
    
    private init() { }
    
//    This value should be stored in AppConfiguration
    private let host: URL = URL(string: "https://vyjidacek.cz/tmap")!

    func requestAPIModel<T: Codable>(_ endpoint: Endpoint) async throws -> T {
        return try await makeAPICall(for: endpoint, of: T.self)
    }
    
    func requestAPIModels<T: Codable>(_ endpoint: Endpoint) async throws -> [T] {
        return try await makeAPICall(for: endpoint, of: [T].self)
    }
}

private extension APIClient {
    
    func createUrlRequest(for endpoint: Endpoint) -> URLRequest? {
        guard let url = createUrl(for: endpoint) else { return nil }
            
        var request = URLRequest(url: url)
        request.httpMethod = "GET"
        
        return request
    }
    
    func createUrl(for endpoint: Endpoint) -> URL? {
        let baseUrl = self.host.appending(path: endpoint.path)
        var urlComponents = URLComponents(url: baseUrl, resolvingAgainstBaseURL: false)
        urlComponents?.queryItems = endpoint.parameters.map { .init(name: $0.key, value: $0.value) }
        
        return urlComponents?.url
    }
    
    func makeAPICall<T: Codable>(for endpoint: Endpoint, of type: T.Type) async throws -> T {
        guard let request = createUrlRequest(for: endpoint) else { throw APIClientError.wrongUrl }
        
        let (data, response) = try await URLSession.shared.data(for: request)
        
        if let httpResponse = response as? HTTPURLResponse {
//            Zde je dobre overit status code, zda vse probehlo jak ma.
//            Prehled kodu: https://en.wikipedia.org/wiki/List_of_HTTP_status_codes
            print(httpResponse.statusCode)
        }
        
        let decoder = JSONDecoder()
        let object = try decoder.decode(type.self, from: data)
        return object
    }
}
