//
//  ContentView.swift
//  NotesSwifUI
//
//  Created by Roman Vyjídáček on 04.04.2023.
//

import SwiftUI
import Foundation

struct NotesListView: View {
    
    let viewModel: NotesListViewModel
    @ObservedObject var state: NotesListState
    
    var body: some View {
        Group {
            if state.notes.isEmpty {
                emptyView
                    .onAppear {
                        viewModel.loadData()
                    }
            } else if state.isLoadingData {
                VStack(spacing: 24) {
                    ProgressView()
                        .progressViewStyle(.circular)
                    
                    Text("Loading Notes")
                        .foregroundColor(.gray)
                }
            } else {
                notesList
            }
        }
        .navigationTitle("Notes")
    }
    
    private var notesList: some View {
        List(state.notes) { note in
            NoteListRow(title: note.title,
                        content: note.content)
        }
        .listStyle(.insetGrouped)
    }
    
    private var emptyView: some View {
        VStack(spacing: 24) {
            Image(systemName: "message.fill")
                .resizable()
                .scaledToFit()
                .frame(width: 80)
                .foregroundColor(.gray.opacity(0.4))
            
            Text("No Content")
                .font(.largeTitle)
            
            Text("You don't have any notes. Use reload button to download data from the server.")
                .padding(horizontal: 10)
                .multilineTextAlignment(.center)
            
            Button("Reload") {
                viewModel.loadData()
            }
            .buttonStyle(.borderedProminent)
        }
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        let viewModel = NotesListViewModel()
        viewModel.state.notes = [] // .mock
        
        return NavigationView {
            NotesListView(viewModel: viewModel,
                          state: viewModel.state)
        }
    }
}

extension Array where Element == NoteDomainModel {
    
    static var mock: [NoteDomainModel] {
        (0...10).map { id in
                .init(id: id,
                      title: "Note \(id)",
                      content: .loremIpsum)
        }
    }
}
