//
//  Repository.swift
//  NotesSwifUI
//
//  Created by Roman Vyjídáček on 06.04.2023.
//

import Foundation
import CoreData

protocol InitiableFromAPIModel {

    associatedtype Model: Codable
    
    init(from apiModel: Model)
}

class Repository {
        
    func fetchRecord<API: Codable, Domain: InitiableFromAPIModel>(_ endpoint: Endpoint) async throws -> Domain where Domain.Model == API {
        let apiModel: API = try await APIClient.shared.requestAPIModel(endpoint)
        return Domain(from: apiModel)
    }
    
    func fetchRecords<API: Codable, Domain: InitiableFromAPIModel>(_ endpoint: Endpoint) async throws -> [Domain] where Domain.Model == API {
        let apiModel: [API] = try await APIClient.shared.requestAPIModels(endpoint)
        return apiModel.map { Domain(from: $0) }
    }
}
